package com.me.brickbuster.layout;

import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.utils.Array;
import com.me.brickbuster.entity.Brick;
import com.me.brickbuster.entity.BrickShape;
import com.me.brickbuster.entity.BrickType;
import com.me.brickbuster.entity.powerup.PowerUpType;
import com.me.brickbuster.state.PlayState;

public class GridLayout implements BrickLayout {

	private final PlayState state;
	private final int cols;
	private final int rows;
	private final float powerUpChance;
	private final float brick_padding;

	private BrickShape shape = null;
	private BrickType type = null;

	private boolean randomShape = false;
	private boolean randomType = false;

	private final Array<Brick> bricks;

	public GridLayout(PlayState state, int cols, int rows, float powerUpChance) {
		this(state, cols, rows, powerUpChance, false, false);
	}

	public GridLayout(PlayState state, int cols, int rows, float powerUpChance, boolean randomShape, boolean randomType) {
		this(state, cols, rows, powerUpChance, BrickShape.RECTANGLE, BrickType.STANDARD_10);
		this.randomShape = randomShape;
		this.randomType = randomType;
	}

	public GridLayout(PlayState state, int cols, int rows, float powerUpChance, BrickShape shape, BrickType type) {
		this.state = state;
		this.cols = cols;
		this.rows = rows;
		this.brick_padding = (PlayState.BOARD_WIDTH - cols * Brick.BRICK_WIDTH) / (cols + 1);
		this.powerUpChance = powerUpChance;
		this.shape = shape;
		this.type = type;
		this.bricks = new Array<Brick>(cols * rows);
	}

	@Override
	public void initialize() {
		for (int col = 0; col < cols; col++) {
			for (int row = 0; row < rows; row++) {
				bricks.add(getBrickForCell(row, col));
			}
		}
	}

	protected Brick getBrickForCell(int row, int col) {
		final float x = brick_padding + Brick.BRICK_WIDTH/2 + (col * (Brick.BRICK_WIDTH + brick_padding));
		final float y = brick_padding + Brick.BRICK_HEIGHT/2 + (row * (Brick.BRICK_HEIGHT + brick_padding));

		PowerUpType powerUpType = null;
		if (MathUtils.randomBoolean(powerUpChance)) {
			powerUpType = PowerUpType.getWeightedRandom();
		}

		if (randomType) {
			type = BrickType.values()[MathUtils.random(BrickType.values().length-1)];
		}

		if (randomShape) {
			shape = BrickShape.values()[MathUtils.random(BrickShape.values().length-1)];
		}

		return new Brick(state, type, shape, powerUpType, x, PlayState.BOARD_HEIGHT - y);
	}

	@Override
	public Array<Brick> getBricks() {
		return bricks;
	}

}
